require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require_relative 'clumsy_player'
require_relative 'berserk_player'

module StudioGame

  class Game
    attr_reader :title
  
    def initialize(title)
      @title = title
      @players = []
    end
  
    def load(from_file)
      File.readlines(from_file).each do |line|
        add_player(Player.from_csv(line))
      end
    end
  
    def save(to_file="player_rankings.csv")
      File.open(to_file, "w") do |file|
        @players.sort.each do |player|
          file.puts player.to_csv
        end
      end
    end
  
    def add_player(player)
      @players << player
    end
  
    def print_name_and_health(player)
      puts "#{player.name} #{player.health}"
    end
  
    def play(turns)
      puts "\nThere are #{@players.size} players in #{@title}:"
  
      @players.each do |player|
        puts player
      end
    
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
    
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end
    
      1.upto(turns) do |turn|
        puts "\nRound: #{turn}"
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end
    end
  
    def total_points
      @players.reduce(0) { |sum, player| sum + player.points}
    end
  
    def print_stats
      puts "\n#{@title} Stats:"
    
      strong, weak = @players.partition { |p| p.strong? }
    
      puts "\nStrong:"
      strong.each do |player|
        print_name_and_health(player)
      end
    
      puts "\nWeak:"
      weak.each do |player|
        print_name_and_health(player)
      end
    
      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts "#{player.name}".ljust(20, '.') + " #{player.score}"
      end

      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "\n#{player.points} grand total points"
      end
    
      puts "\nTotal Game Points:\n#{total_points}"
    end
  end
end