require_relative 'playable'

module StudioGame
  
  class Player
    include Playable
  
    attr_accessor :name
    attr_accessor :health
  
    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end
  
    def self.from_csv(line)
      name, health = line.split(",")
      Player.new(name, Integer(health))
    end
  
    def to_csv
      "#{@name}, #{points}"
    end
  
    def name=(new_name)
      @name = new_name.capitalize
    end
  
    # virtual attribute - derived from existing instance variables
    def score
      @health + points
    end

    def <=>(other_player)
      other_player.score <=> score
    end

    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{@name}'s treasures: #{@found_treasures}"
    end
  
    def each_found_treasure
      @found_treasures.each do |name, points|
        treasure = Treasure.new(name, points)
        yield treasure
      end
    end

    def points
      @found_treasures.values.reduce(0, :+) 
    end

    def to_s
      "I'm #{@name} with health = #{health} (#{status}), points = #{points}, and score = #{score}."
    end
  end
end

if __FILE__ == $0
  player1 = Player.new("larry", 60)
  puts player1
  
  player1.w00t
  player1.w00t
  puts player1.health
  
  player1.blam
  puts player1.health
  puts player1
end